import {createClient} from './client';
import * as config from './config';
import {addMessageEventsListener} from './events';
import {ForumChannel} from 'discord.js';
import {setupThreads} from './modules/thread';
import {BotContext} from './common';
import * as logger from './log';

interface Params {
	logger?: Pick<typeof console, 'log' | 'error' | 'warn'>;
	config: typeof config;
}

const main = async (params = {} as Params) => {
	const { logger: l = logger, config: c, } = params
	const client = createClient({
		intents: c.meta.intentsInteger,
	});

	const context: BotContext = {
		channel: undefined,
		onMessageCreate: (fn) => {
			client.on('messageCreate', fn);
		},
		onThreadCreate: (fn) => {
			client.on('threadCreate', fn);
		},
		fetchChannel: (channelId: string) => client.channels.fetch(channelId),
	};

	client.on('ready', async () => {
		if (!client.user) {
			l.error('Not logged in!');
			return;
		}
		l.log(`Logged in as ${client.user.tag}!`);

		const channel = await client.channels.fetch(c.meta.channelId);
		if (!(channel instanceof ForumChannel)) {
			l.error(`Channel ${c.meta.channelId} is not a forum!`);
			return;
		}

		context.channel = channel;
		l.log('Syncing up threads...');
		await setupThreads(context);
		l.log('Sync done! Now listening for new messages.');
	});

	addMessageEventsListener(context);
	l.log('Logging in to gateway...');
	await client.login(c.meta.token);
}

// TODO load channelId from config
// TODO add discord wizard for chatting with bot for setup (e.g. selecting forum channel ID)
void main({
	logger,
	config,
});
